# Create a very basic animation as an example
# This is a simple, but not the best way of implementing this animation.
#
# This script should work with both python2 and python3

import numpy as np
import matplotlib.pyplot as plt
from matplotlib.animation import FuncAnimation

# Resolution of the video (should be at least 200 for a good quality)
dpi = 400
# Number of time steps, for which the plot is calculated
nt = 500
# Number of points on the x axis
nx = 400
x = np.linspace(-1, 1, nx)

# Calculate a function of x and time, which will be shown in the animation
# The function is a wavefunction of a quantum mechanical particle in a one dimensional box.

y1 = np.cos(np.pi*x/2)
y2 = np.sin(np.pi*x)
y3 = np.cos(3*np.pi*x/2)
y4 = np.sin(2*np.pi*x)
e1 = 0.5
e2 = 1
e3 = 1.5
e4 = 2

# psi(t) returns an array of complex y values.
psi = lambda t: 0.4*np.exp(2j*np.pi*e1*t)*y1 + 0.3*np.exp(2j*np.pi*e2*t)*y2 - 0.2*np.exp(2j*np.pi*e3*t)*y3 - 0.1*np.exp(2j*np.pi*e4*t)*y4

y = psi(0)

# Set up a plot
fig, ax = plt.subplots()
fig.set_size_inches(8/2.54, 5/2.54)
ax.set_xlim(-1, 1)
ax.set_ylim(0, 0.85)

# Show the phase of the complex values in psi(t) as a color plot
img = ax.imshow(np.angle(y).reshape((1,nx)), cmap=plt.cm.hsv, norm=plt.Normalize(-np.pi,np.pi), extent=[-np.pi,np.pi,0,0.85])

# Clip this color plot, such that it is only shown in the area defined by 0 < y < abs( psi(t) )
fill, = ax.fill(x, np.abs(y), facecolor='none')
img.set_clip_path(fill)

# Plot the absolute value of psi as a black line
plot, = ax.plot(x, np.abs(y), color='black')

# Set aspect ratio
ax.set_aspect(1.5)

# Save the plot to an image
fig.savefig("image.jpg", dpi=dpi)

# Define a function which updates the data
def calc_data(t):
    y = psi(t)
    fill.set_xy( np.array((x, np.abs(y))).T )
    img.set_data(np.angle(y).reshape((1, nx)))
    img.set_clip_path(fill)
    plot.set_ydata(np.abs(y))

# Define an array of times, for which the plot will be shown
times = np.linspace(0, 2*np.pi, nt, endpoint=False)
# Create an animation
anm = FuncAnimation(fig, calc_data, times)
# Save the animation
anm.save("movie.mp4", fps=50, dpi=dpi)
